/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.programmation-facile.com/
  Portions created by the Initial Developer are Copyright (C) 2013
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */
package com.formation.actionscript.observateur 
{
	
	/**
	 * Le mécanisme de diffusion des événements utilisé par le Sujet.
	 * @author Matthieu
	 * 
	 */
	public class Observateur 
	{
		protected var sName:String = "";// le nom du sujet - accessible par les classes filles (extends)
		
		private var _bChanged : Boolean = false; // si le sujet a changé ou non.
		private var _aEcouteurs : Array;// stockage des observateurs (ou écouteurs)
		

		/**
		 * Constructeur.
		 * 
		 */
		public function Observateur ()
		{
			_aEcouteurs = [];
		}
		


		/**
		 * Renvoie le nom de l'éditeur / du journal
		 */
		public function getName() : String
		{
			return sName;
		}
				
		
		/**
		 * Ajoute un écouteur ou observateur.
		 * 
		 */
		public function addListener (oListener : IObservateur) : Boolean
		{
			if (oListener == null)	return false;

			var nNumberObserv : Number = _aEcouteurs.length;
			
			// autorise qu'une seule occurence d'un écouteur.
			// pour éviter les doubles notifications.
			for (var i : Number = 0; i < nNumberObserv; i ++)
				if (_aEcouteurs [i] == oListener)	return false;

			_aEcouteurs.push (oListener);
			return true;
		}
		
		
		/**
		 * Supprime un écouteur ou observateur.
		 * 
		 */
		public function removeListener (oListener : IObservateur) : Boolean
		{
			var nNumberObserv : Number = _aEcouteurs.length;
			for (var i : Number = 0; i < nNumberObserv; i ++)
			{
				// suppression de l'écouteur dès qu'il est trouvé
				if (_aEcouteurs [i] == oListener)
				{
					_aEcouteurs.splice (i, 1);
					return true;
				}
			}
			return false;
		}
		

		/**
		 * Informe les écouteurs d'un changement d'état du sujet.
		 * 
		 */
		public function notifyListener (oInfo : Object) : void
		{
			// pas d'appel si le sujet n'a pas changé
			if ( ! _bChanged)	return;
			
			// copie du tableau d'écouteurs, pour notifier que ceux actuellement inscrits.
			var _aObservateursCopy : Array = _aEcouteurs.slice (0);
			clearChanged ();
			
			var nNumberObserv : Number = _aObservateursCopy.length;
			for (var i : Number = nNumberObserv - 1; i >= 0; i --)
				// appel la fonction que doivent implementer les écouteurs
				_aObservateursCopy [i].update (this, oInfo);

		}
		
		
		/**
		 * Supprime tous les écouteurs.
		 * 
		 */
		public function clearListeners () : void
		{
			_aEcouteurs = [];
		}
		
		
		/**
		 * L'état du sujet a changé.
		 * 
		 */
		protected function setChanged () : void
		{
			_bChanged = true;
		}
		
		protected function clearChanged () : void
		{
			_bChanged = false;
		}
		
		
		/**
		 * Renvoie si le sujet a changé d'état.
		 * 
		 */
		public function hasChanged () : Boolean
		{
			return _bChanged;
		}
		
		
		/**
		 * Renvoie le nombre d'écouteurs.
		 * 
		 */
		public function getNumberListeners () : Number
		{
			return _aEcouteurs.length;
		}
		
	}
}
